
// Calculate filter coefficients for a 4th-order Linkwitz-Riley pair of filters to implement a crossover.
//
// This filter:
//
// 1. Has absolutely flat amplitude response throughout the passband with a 6 dB/octave rolloff rate after the crossover point.
// 2. The acoustic sum of the two filter responses is unity at crossover. 
//    (Amplitude response of each is -3 dB at crossover, i.e., there is no peaking in the summed acoustic output.)
// 3. The low pass and high pass outputs are everywhere in phase, including at the crossover point.
//
//	Filter equations originally from:
//
// http://www.planetanalog.com/article/printableArticle.jhtml?articleID=12802683
//  By Rusty Allred, Texas Instruments, Dallas
//  July 01, 2003
//
// Later adapted for Jamoma by Trond Lossius:
//
// https://github.com/jamoma/JamomaCore/blob/master/DSP/extensions/FilterLib/source/TTLowpassLinkwitzRiley4.cpp
// by Trond Lossius
// Copyright 2008
//
// Adapted here for Javascript by Timothy Place
// September 2016


outlets = 4;


function msg_float(frequency_in_hz) {
	var fs = 44100.0;
	var frequency_in_radians = Math.PI * 2.0 * frequency_in_hz;
	var k = frequency_in_radians / Math.tan(Math.PI * frequency_in_hz / fs);
	var sqrt_of_two = Math.sqrt(2.0);
	var temp = 4 * Math.pow(frequency_in_radians,2) * Math.pow(k, 2)  + 2 * sqrt_of_two * Math.pow(frequency_in_radians, 3) * k + Math.pow(k, 4) + 2 * sqrt_of_two * frequency_in_radians * Math.pow(k, 3) + Math.pow(frequency_in_radians, 4);

	var a0 = Math.pow(frequency_in_radians, 4)	/	temp;	
	var a1 = 4 * Math.pow(frequency_in_radians, 4)	/	temp;
	var a2 = 6 * Math.pow(frequency_in_radians, 4)	/	temp;
	var a3 = a1;
	var a4 = a0;
	
	a0_high = Math.pow(k,4) / temp;
	a1_high = -4 * Math.pow(k,4) / temp;
	a2_high = 6 * Math.pow(k,4) / temp;
	a3_high = a1_high;
	a4_high = a0_high;

	var b1 = (
		4 * Math.pow(frequency_in_radians,4)
		+ 4 * sqrt_of_two * Math.pow(frequency_in_radians,3) * k
		- 4 * Math.pow(k, 4)
		- 4 * sqrt_of_two * frequency_in_radians * Math.pow(k,3)
	) / temp;

	var b2 = (
		6 * Math.pow(frequency_in_radians,4)
		- 8 * Math.pow(frequency_in_radians,2) * Math.pow(k,2)
		+ 6 * Math.pow(k, 4)
	) /	temp;

	var b3 = (
		-4 * sqrt_of_two * Math.pow(frequency_in_radians,3) * k
		+ 4 * Math.pow(frequency_in_radians, 4)
		+ 4 * sqrt_of_two * frequency_in_radians * Math.pow(k,3)
		- 4 * Math.pow(k, 4)
	) / temp;

	var b4 = (
		Math.pow(k,4)
		- 2 * sqrt_of_two * Math.pow(frequency_in_radians, 3) * k
		+ Math.pow(frequency_in_radians, 4)
		- 2 * sqrt_of_two * frequency_in_radians * Math.pow(k,3)
		+ 4 * Math.pow(frequency_in_radians,2) * Math.pow(k,2)
	) / temp;	

	outlet(3, 1.0, b1, b2, b3, b4);
	outlet(2, a0_high, a1_high, a2_high, a3_high, a4_high);
	outlet(1, 1.0, b1, b2, b3, b4);
	outlet(0, a0, a1, a2, a3, a4);
}
